"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseApi = exports.parse = exports.AssessmentStandardCode = exports.AssessmentFreeTextAuto = exports.AssessmentFillInTheBlanks = exports.AssessmentFreeText = exports.AssessmentGradeBook = exports.AssessmentMultipleChoice = exports.AssessmentAdvanced = exports.AssessmentParsons = exports.Assessment = exports.BLOOMS_LEVEL = exports.API_HASH_TAG = exports.API_ID_TAG = void 0;
const lodash_1 = __importDefault(require("lodash"));
const object_hash_1 = __importDefault(require("object-hash"));
const uuid_1 = require("uuid");
const fs_1 = __importDefault(require("fs"));
const crypto_1 = __importDefault(require("crypto"));
const tools_1 = require("./tools");
const path_1 = __importDefault(require("path"));
exports.API_ID_TAG = 'CODIO_API_ID';
exports.API_HASH_TAG = 'CODIO_API_HASH';
const FROM_LIBRARY_DUMMY = '<<<<<library-assessment>>>>>';
exports.BLOOMS_LEVEL = {
    '1': 'Level I - Remembering',
    '2': 'Level II - Understanding',
    '3': 'Level III - Applying',
    '4': 'Level IV - Analyzing',
    '5': 'Level V - Evaluating',
    '6': 'Level VI - Creating'
};
function fixGuidance(json) {
    if (!lodash_1.default.isUndefined(json.source.showGuidanceAfterResponse)) {
        // old format
        return {
            type: json.source.showGuidanceAfterResponse ? "Always" : "Never"
        };
    }
    return json.source.showGuidanceAfterResponseOption;
}
function getContentForComplexAssessment(taskId, content) {
    const pattern = new RegExp('{.+\\|assessment}\\(' + taskId + '\\)\\n?', 'gm');
    const result = content.replace(pattern, FROM_LIBRARY_DUMMY + '\n');
    return result.replace(/{.+\|assessment}\(.+-[0-9]+\)\n?/gm, '');
}
function getButtonTextForComplexAssessment(taskId, type, content) {
    const pattern = new RegExp('{(.+?)\\|assessment}\\(' + taskId + '\\)\\n?', 'gm');
    const match = pattern.exec(content);
    return match ? match[1] : undefined;
}
function findPage(taskId, metadata) {
    for (const { page, data } of metadata) {
        if (page.match(taskId)) {
            return { page, data };
        }
    }
    throw new Error(`Assessment ${taskId} is not in use`);
}
function extractLayout(json, metadata) {
    if (!lodash_1.default.isUndefined(json.source.metadata.layout)) {
        // already has layout
        return {
            layout: json.source.metadata.layout,
            content: json.source.metadata.content,
            pageTitle: json.source.metadata.pageTitle,
            buttonText: json.source.metadata.buttonText
        };
    }
    // parse layout
    const { page, data } = findPage(json.taskId, metadata);
    if (data.layout === '1-panel') {
        // simple do not need alyout
        return {
            layout: undefined,
            content: undefined,
            pageTitle: undefined,
            buttonText: undefined,
        };
    }
    return {
        layout: data.layout,
        content: getContentForComplexAssessment(json.taskId, page),
        pageTitle: data.title,
        buttonText: getButtonTextForComplexAssessment(json.taskId, json.type, page)
    };
}
class Assessment {
    constructor(json, metadata) {
        this.type = 'None';
        this.basePath = '';
        this.taskId = 'none';
        this.body = {};
        if (!metadata) {
            this.assessmentId = json.assessmentId;
            this.details = json.details;
            this.metadata = {
                files: json.metadata.files,
                opened: json.metadata.opened,
                layout: json.metadata.layout,
                content: json.metadata.content,
                pageTitle: json.metadata.pageTitle,
                buttonText: json.metadata.buttonText,
                tags: this.getTagsFromJsonApi(json.metadata.tags)
            };
        }
        else {
            this.details = {
                name: json.source.name,
                showName: json.source.showName,
                instructions: json.source.instructions,
                showExpectedAnswer: json.source.showExpectedAnswer,
                guidance: json.source.guidance,
                maxAttemptsCount: getMaxAttemptsCount(json)
            };
            const tags = this.getTagsFromJson(json.source.metadata.tags);
            tags.set('Learning Objectives', json.source.learningObjectives);
            tags.set(`Bloom's level`, exports.BLOOMS_LEVEL[json.source.bloomsObjectiveLevel]);
            const { layout, content, pageTitle, buttonText } = extractLayout(json, metadata);
            this.metadata = {
                tags,
                layout,
                content,
                pageTitle,
                buttonText,
                files: json.source.metadata.files,
                opened: json.source.metadata.opened
            };
            this.taskId = json.taskId;
        }
        // assessment has id already => not new update
        this.getId();
    }
    getTagsFromJson(json) {
        const tags = new Map();
        for (const tag of json) {
            tags.set(tag.name, tag.value);
        }
        return tags;
    }
    getTagsFromJsonApi(json) {
        const array = Object.entries(json);
        return new Map(array);
    }
    getBundle(base) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.metadata.files.length === 0) {
                return;
            }
            return (0, tools_1.createTar)(base, this.metadata.files);
        });
    }
    getId() {
        if (!this.metadata.tags.has(exports.API_ID_TAG)) {
            this.metadata.tags.set(exports.API_ID_TAG, (0, uuid_1.v4)());
        }
        return this.metadata.tags.get(exports.API_ID_TAG) || '';
    }
    loadFiles(basePath) {
        const hashes = this.metadata.files.map(filePath => {
            try {
                const fullPath = path_1.default.join(basePath, filePath);
                const fileBuffer = fs_1.default.readFileSync(fullPath);
                const hashSum = crypto_1.default.createHash('md5');
                hashSum.update(fileBuffer);
                const hex = hashSum.digest('hex');
                return {
                    hex,
                    filePath
                };
            }
            catch (error) {
                console.error(`Error Processing ${filePath} problem: ${error.message}`);
                return {
                    hex: '',
                    filePath
                };
            }
        });
        return hashes;
    }
    getHash() {
        const object = this._export(true);
        object['filesHash'] = this.loadFiles(this.basePath);
        return (0, object_hash_1.default)(object);
    }
    _export(checksum = false) {
        const tags = {};
        this.metadata.tags.forEach((value, name) => {
            if (checksum && (name == exports.API_HASH_TAG || name == exports.API_ID_TAG)) {
                // skip for hashing
                return;
            }
            tags[name] = value;
        });
        if (!checksum) {
            tags[exports.API_HASH_TAG] = this.getHash();
        }
        return {
            details: this.details,
            body: this.body,
            metadata: {
                files: this.metadata.files,
                opened: this.metadata.opened,
                layout: this.metadata.layout,
                content: this.metadata.content,
                pageTitle: this.metadata.pageTitle,
                buttonText: this.metadata.buttonText,
                tags,
            }
        };
    }
    export(checksum = false) {
        return JSON.stringify(this._export(checksum));
    }
}
exports.Assessment = Assessment;
class AssessmentParsons extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Parsons Puzzle';
        if (metadata) {
            this.body = {
                parsonPuzzle: {
                    initial: json.source.initial,
                    options: json.source.options,
                    grader: json.source.grader,
                    showGuidanceAfterResponseOption: fixGuidance(json),
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentParsons = AssessmentParsons;
class AssessmentAdvanced extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Advanced Code Test';
        if (metadata) {
            this.body = {
                codeTest: {
                    command: json.source.command,
                    timeoutSeconds: json.source.timeoutSeconds,
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    showGuidanceAfterResponseOption: fixGuidance(json)
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentAdvanced = AssessmentAdvanced;
class AssessmentMultipleChoice extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Multiple Choice';
        if (metadata) {
            const options = json.source.answers.map((rec) => {
                return {
                    id: rec._id,
                    correct: rec.correct,
                    answer: rec.answer
                };
            });
            this.body = {
                multipleChoice: {
                    options,
                    isMultipleResponse: json.source.multipleResponse,
                    isRandomized: json.source.isRandomized,
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    showGuidanceAfterResponseOption: fixGuidance(json)
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentMultipleChoice = AssessmentMultipleChoice;
class AssessmentGradeBook extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Grade Book';
        if (metadata) {
            this.body = {
                gradeBook: {
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    rubrics: json.source.rubrics,
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentGradeBook = AssessmentGradeBook;
class AssessmentFreeText extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Free Text';
        if (metadata) {
            this.body = {
                freeText: {
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    showGuidanceAfterResponseOption: fixGuidance(json),
                    rubrics: json.source.rubrics,
                    previewType: json.source.previewType
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentFreeText = AssessmentFreeText;
class AssessmentFillInTheBlanks extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Fill in the Blanks';
        if (metadata) {
            const texts = json.source.tokens.text;
            texts.unshift('');
            this.body = {
                fillInBlanks: {
                    text: json.source.text,
                    showValues: json.source.showValues,
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    blanks: json.source.tokens.blank,
                    texts: lodash_1.default.filter(texts, lodash_1.default.isString),
                    distractors: json.source.distractors,
                    showGuidanceAfterResponseOption: fixGuidance(json)
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentFillInTheBlanks = AssessmentFillInTheBlanks;
class AssessmentFreeTextAuto extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Free Text Autograde';
        if (metadata) {
            this.body = {
                freeTextAuto: {
                    arePartialPointsAllowed: json.source.arePartialPointsAllowed,
                    showGuidanceAfterResponseOption: fixGuidance(json),
                    previewType: json.source.previewType,
                    command: json.source.command,
                    timeout: json.source.timeoutSeconds,
                }
            };
        }
        else {
            this.body = json.body;
        }
    }
}
exports.AssessmentFreeTextAuto = AssessmentFreeTextAuto;
class AssessmentStandardCode extends Assessment {
    constructor(json, metadata) {
        super(json, metadata);
        this.type = 'Standard Code Test';
        if (!metadata) {
            this.body = json.body;
        }
        else {
            this.body = {
                codeCompare: {
                    options: json.source.options,
                    command: json.source.command,
                    preExecuteCommand: json.source.preExecuteCommand,
                    showGuidanceAfterResponseOption: fixGuidance(json),
                    sequence: json.source.sequence
                }
            };
        }
    }
}
exports.AssessmentStandardCode = AssessmentStandardCode;
function getMaxAttemptsCount(json) {
    if (json.source.maxAttemptsCount) {
        return json.source.maxAttemptsCount;
    }
    if (json.source.oneTimeTest ||
        json.type === 'multiple-choice' ||
        json.type === 'fill-in-the-blanks') {
        return 1;
    }
    return 0;
}
function parse(json, metadataPages) {
    switch (json.type) {
        case 'test':
            return new AssessmentAdvanced(json, metadataPages);
        case 'multiple-choice':
            return new AssessmentMultipleChoice(json, metadataPages);
        case 'fill-in-the-blanks':
            return new AssessmentFillInTheBlanks(json, metadataPages);
        case 'code-output-compare':
            return new AssessmentStandardCode(json, metadataPages);
        case 'parsons-puzzle':
            return new AssessmentParsons(json, metadataPages);
        case 'free-text':
            return new AssessmentFreeText(json, metadataPages);
        case 'free-text-auto':
            return new AssessmentFreeTextAuto(json, metadataPages);
        case 'grade-book':
            return new AssessmentGradeBook(json, metadataPages);
        default:
            throw new Error('assessemnt type not found');
    }
}
exports.parse = parse;
function parseApi(json) {
    const type = json.metadata.tags['Assessment Type'];
    switch (type) {
        case 'Advanced Code Test':
            return new AssessmentAdvanced(json);
        case 'Multiple Choice':
            return new AssessmentMultipleChoice(json);
        case 'Fill in the Blanks':
            return new AssessmentFillInTheBlanks(json);
        case 'Standard Code Test':
            return new AssessmentStandardCode(json);
        case 'Parsons Puzzle':
            return new AssessmentParsons(json);
        case 'Free Text':
            return new AssessmentFreeText(json);
        case 'Free Text Autograde':
            return new AssessmentFreeTextAuto(json);
        case 'Grade Book':
            return new AssessmentGradeBook(json);
        default:
            throw new Error(`assessment type ${type} not found`);
    }
}
exports.parseApi = parseApi;
